//
// Created by dell on 24-9-23.
//

#include "usonic_driver.h"
#include "gd32e23x.h"
#include "systick.h"

uint8_t speed_pwm = 30;              //bldc default pwm is 30

void led_config(void)
{
    rcu_periph_clock_enable(RCU_GPIOA);

    gpio_mode_set(GPIOA, GPIO_MODE_OUTPUT, GPIO_PUPD_NONE, GPIO_PIN_9);
    gpio_output_options_set(GPIOA, GPIO_OTYPE_PP, GPIO_OSPEED_50MHZ, GPIO_PIN_9);
    gpio_bit_write(GPIOA, GPIO_PIN_9, SET);

    rcu_periph_clock_enable(RCU_TIMER13);
    timer_deinit(RCU_TIMER13);

    timer_parameter_struct timer_initpara;
    timer_struct_para_init(&timer_initpara);
    timer_initpara.prescaler        =7199;
    timer_initpara.alignedmode      =TIMER_COUNTER_EDGE;
    timer_initpara.counterdirection =TIMER_COUNTER_UP;
    timer_initpara.period           =999;
    timer_initpara.clockdivision    =TIMER_CKDIV_DIV1;
    timer_init(TIMER13, &timer_initpara);

    timer_auto_reload_shadow_enable(TIMER13);
    timer_interrupt_enable(TIMER13, TIMER_INT_UP);
    nvic_irq_enable(TIMER13_IRQn, 0);
    timer_enable(TIMER13);
}

void usart_config(void)
{
    rcu_periph_clock_enable(RCU_GPIOA);
    rcu_periph_clock_enable(RCU_USART0);

    gpio_af_set(GPIOA, GPIO_AF_1, GPIO_PIN_2 | GPIO_PIN_3);

    /* configure USART Tx as alternate function push-pull */
    gpio_mode_set(GPIOA, GPIO_MODE_AF, GPIO_PUPD_PULLUP, GPIO_PIN_2);
    gpio_output_options_set(GPIOA, GPIO_OTYPE_PP, GPIO_OSPEED_10MHZ, GPIO_PIN_2);

    gpio_mode_set(GPIOA, GPIO_MODE_AF, GPIO_PUPD_PULLUP, GPIO_PIN_3);
    gpio_output_options_set(GPIOA, GPIO_OTYPE_PP, GPIO_OSPEED_10MHZ, GPIO_PIN_3);

    /* USART configure */
    usart_deinit(USART0);
    usart_baudrate_set(USART0, 115200U);
    usart_receive_config(USART0, USART_RECEIVE_ENABLE);
    usart_transmit_config(USART0, USART_TRANSMIT_ENABLE);

    usart_enable(USART0);
}

void ultrasonic_gpio_config(void)
{
    rcu_periph_clock_enable(RCU_GPIOB);

    gpio_mode_set(GPIOB, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO_PIN_1);
    gpio_output_options_set(GPIOB, GPIO_OTYPE_PP, GPIO_OSPEED_50MHZ, GPIO_PIN_1);
    gpio_af_set(GPIOB, GPIO_AF_1, GPIO_PIN_1);
}

void ultrasonic_timer_config(void)
{
    timer_oc_parameter_struct timer_ocinitpara;
    timer_parameter_struct    timer_initpara;

    rcu_periph_clock_enable(RCU_TIMER2);
    timer_deinit(TIMER2);

    timer_struct_para_init(&timer_initpara);
    timer_initpara.prescaler          = 0;
    timer_initpara.alignedmode        = TIMER_COUNTER_EDGE;
    timer_initpara.counterdirection   = TIMER_COUNTER_UP;
    timer_initpara.period             = 239;
    timer_initpara.clockdivision      = TIMER_CKDIV_DIV1;
    timer_init(TIMER2, &timer_initpara);

    timer_channel_output_struct_para_init(&timer_ocinitpara);
    timer_ocinitpara.outputstate  =TIMER_CCX_ENABLE;
    timer_ocinitpara.outputnstate =TIMER_CCXN_DISABLE;
    timer_ocinitpara.ocpolarity   =TIMER_OC_POLARITY_HIGH;
    timer_ocinitpara.ocnpolarity  =TIMER_OCN_POLARITY_HIGH;
    timer_ocinitpara.ocidlestate  =TIMER_OC_IDLE_STATE_LOW;
    timer_ocinitpara.ocnidlestate =TIMER_OCN_IDLE_STATE_LOW;
    timer_channel_output_config(TIMER2, TIMER_CH_3, &timer_ocinitpara);

    timer_channel_output_pulse_value_config(TIMER2, TIMER_CH_3, 120);
    timer_channel_output_mode_config(TIMER2, TIMER_CH_3, TIMER_OC_MODE_PWM0);
    timer_auto_reload_shadow_enable(TIMER2);

    timer_interrupt_enable(TIMER2, TIMER_INT_UP);
    // nvic_irq_enable(TIMER2_IRQn, 1);
    // timer_enable(TIMER2);
}

void ultrasonic_config(void) {
    led_config();
    usart_config();
    ultrasonic_gpio_config();
    ultrasonic_timer_config();
}

void ultrasonic_pwm_out_cycles(uint8_t cycles) {
    uint8_t current_cycle = 0;

    timer_channel_output_pulse_value_config(TIMER2, TIMER_CH_3, 120);
    timer_channel_output_mode_config(TIMER2, TIMER_CH_3, TIMER_OC_MODE_PWM1);
    timer_enable(TIMER2);

    while (current_cycle < cycles)
    {
        while (!timer_interrupt_flag_get(TIMER2, TIMER_INT_FLAG_UP));
        timer_interrupt_flag_clear(TIMER2, TIMER_INT_FLAG_UP);
        current_cycle ++;
    }
    // delay_nop();
    timer_disable(TIMER2);
    // if(gpio_output_bit_get(GPIOB, GPIO_PIN_1) == SET)
    // {
    //     gpio_bit_reset(GPIOB, GPIO_PIN_1);
    // }
}

